@@ -1,15 +1,67 @@
-use anyhow::{anyhow, Result};
+use anyhow::{anyhow, Context, Result};
 use k8s_openapi::apiextensions_apiserver::pkg::apis::apiextensions::v1::{
-    CustomResourceDefinition, CustomResourceDefinitionVersion,
+    CustomResourceDefinition, CustomResourceDefinitionVersion, CustomResourceSubresources,
 };
 use kopium::{analyze, OutputStruct};
 use kube::{api, core::Version, Api, Client, ResourceExt};
 use quote::format_ident;
+use std::path::PathBuf;
 use structopt::{clap, StructOpt};
 
-const KEYWORDS: [&str; 23] = [
-    "for", "impl", "continue", "enum", "const", "break", "as", "move", "mut", "mod", "pub", "ref", "self",
-    "static", "struct", "super", "true", "trait", "type", "unsafe", "use", "where", "while",
+// synced from https://doc.rust-lang.org/reference/keywords.html feb 2022
+const KEYWORDS: [&str; 52] = [
+    "as",
+    "break",
+    "const",
+    "continue",
+    "crate",
+    "else",
+    "enum",
+    "extern",
+    "false",
+    "fn",
+    "for",
+    "if",
+    "impl",
+    "in",
+    "let",
+    "loop",
+    "match",
+    "mod",
+    "move",
+    "mut",
+    "pub",
+    "ref",
+    "return",
+    "self",
+    "Self",
+    "static",
+    "struct",
+    "super",
+    "trait",
+    "true",
+    "type",
+    "unsafe",
+    "use",
+    "where",
+    "while",
+    "async",
+    "await",
+    "dyn",
+    "abstract",
+    "become",
+    "box",
+    "do",
+    "final",
+    "macro",
+    "override",
+    "priv",
+    "typeof",
+    "unsized",
+    "virtual",
+    "yield",
+    "try",
+    "macro_rules",
 ];
 
 #[derive(StructOpt, Debug)]
@@ -19,20 +71,37 @@ const KEYWORDS: [&str; 23] = [
     about = "Kubernetes OPenapI UnMangler",
 )]
 struct Kopium {
-    #[structopt(about = "Give the name of the input CRD to use e.g. prometheusrules.monitoring.coreos.com")]
+    #[structopt(
+        about = "Give the name of the input CRD to use e.g. prometheusrules.monitoring.coreos.com",
+        conflicts_with("file")
+    )]
     crd: Option<String>,
+
+    #[structopt(
+        parse(from_os_str),
+        about = "Point to the location of a CRD to use on disk",
+        long = "--filename",
+        short = "f",
+        conflicts_with("crsd")
+    )]
+    file: Option<PathBuf>,
+
     #[structopt(about = "Use this CRD version if multiple versions are present", long)]
     api_version: Option<String>,
+
     #[structopt(about = "Do not emit prelude", long)]
     hide_prelude: bool,
+
     #[structopt(about = "Emit doc comments from descriptions", long)]
     docs: bool,
+
     #[structopt(
         about = "Derive these extra traits on generated structs",
         long,
         possible_values = &["Copy", "Default", "PartialEq", "Eq", "PartialOrd", "Ord", "Hash"],
     )]
     derive: Vec<String>,
+
     #[structopt(subcommand)]
     command: Option<Command>,
 }
@@ -51,44 +120,50 @@ enum Command {
 #[tokio::main]
 async fn main() -> Result<()> {
     env_logger::init();
-
     Kopium::from_args().dispatch().await
 }
 
 impl Kopium {
     async fn dispatch(&self) -> Result<()> {
-        let api = Client::try_default()
-            .await
-            .map(Api::<CustomResourceDefinition>::all)?;
         if let Some(name) = self.crd.as_deref() {
-            if self.command.is_none() {
-                self.generate(api, name).await
-            } else {
-                self.help()
-            }
+            let api = Client::try_default()
+                .await
+                .map(Api::<CustomResourceDefinition>::all)?;
+            let crd = api.get(name).await?;
+            self.generate(crd).await
+        } else if let Some(f) = self.file.as_deref() {
+            // no cluster access needed in this case
+            let data =
+                std::fs::read_to_string(&f).with_context(|| format!("Failed to read {}", f.display()))?;
+            let crd: CustomResourceDefinition = serde_yaml::from_str(&data)?;
+            self.generate(crd).await
         } else {
             match self.command {
-                Some(Command::ListCrds) => self.list_crds(api).await,
+                Some(Command::ListCrds) => {
+                    let api = Client::try_default()
+                        .await
+                        .map(Api::<CustomResourceDefinition>::all)?;
+                    self.list_crds(api).await
+                }
                 Some(Command::Completions { shell }) => self.completions(shell),
                 None => self.help(),
             }
         }
     }
 
-    async fn generate(&self, api: Api<CustomResourceDefinition>, name: &str) -> Result<()> {
-        let crd = api.get(name).await?;
+    async fn generate(&self, crd: CustomResourceDefinition) -> Result<()> {
         let version = self.api_version.as_deref();
         let version = find_crd_version(&crd, version)?;
         let data = version
             .schema
             .as_ref()
             .and_then(|schema| schema.open_api_v3_schema.clone());
-        let version = version.name.clone();
+        let version_name = version.name.clone();
 
-        let kind = crd.spec.names.kind;
-        let plural = crd.spec.names.plural;
-        let group = crd.spec.group;
-        let scope = crd.spec.scope;
+        let kind = &crd.spec.names.kind;
+        let plural = &crd.spec.names.plural;
+        let group = &crd.spec.group;
+        let scope = &crd.spec.scope;
 
         if let Some(schema) = data {
             let mut structs = vec![];
@@ -108,11 +183,16 @@ impl Kopium {
                         self.print_derives();
                         println!(
                             r#"#[kube(group = "{}", version = "{}", kind = "{}", plural = "{}")]"#,
-                            group, version, kind, plural
+                            group, version_name, kind, plural
                         );
                         if scope == "Namespaced" {
                             println!(r#"#[kube(namespaced)]"#);
                         }
+
+                        if let Some(CustomResourceSubresources { status: Some(_), .. }) = version.subresources
+                        {
+                            println!(r#"#[kube(status = "{}Status")]"#, kind);
+                        }
                         // don't support grabbing original schema atm so disable schemas:
                         // (we coerce IntToString to String anyway so it wont match anyway)
                         println!(r#"#[kube(schema = "disabled")]"#);
@@ -140,7 +220,7 @@ impl Kopium {
                 }
             }
         } else {
-            log::error!("no schema found for crd {}", name);
+            log::error!("no schema found for crd");
         }
 
         Ok(())
@@ -171,7 +251,7 @@ impl Kopium {
         // print doc strings if requested in arguments
         if self.docs {
             if let Some(d) = doc {
-                println!("{}/// {}", indent, d);
+                println!("{}/// {}", indent, d.replace("\n", &format!("\n{}/// ", indent)));
                 // TODO: logic to split doc strings by sentence / length here
             }
         }
@@ -189,7 +269,6 @@ impl Kopium {
     }
 }
 
-
 fn print_prelude(results: &[OutputStruct]) {
     println!("use kube::CustomResource;");
     println!("use serde::{{Serialize, Deserialize}};");
@@ -202,10 +281,12 @@ fn print_prelude(results: &[OutputStruct]) {
     if results.iter().any(|o| o.uses_date()) {
         println!("use chrono::naive::NaiveDate;");
     }
+    if results.iter().any(|o| o.uses_int_or_string()) {
+        println!("use k8s_openapi::apimachinery::pkg::util::intstr::IntOrString;");
+    }
     println!();
 }
 
-
 fn find_crd_version<'a>(
     crd: &'a CustomResourceDefinition,
     version: Option<&str>,
